@@ -1,3 +1,4 @@
+/* vi: set expandtab sw=4 sts=4: */
 /* file_util.h - convenience routines for common file operations
 
    Carl D. Worth
@@ -22,19 +23,26 @@
 extern "C" {
 #endif
 
+struct stat;
+
+int xlstat(const char *file_name, struct stat *st);
 int file_exists(const char *file_name);
 int file_is_dir(const char *file_name);
 int file_is_symlink(const char *file_name);
-char *file_read_line_alloc(FILE *file);
-int file_move(const char *src, const char *dest);
+char *file_readlink_alloc(const char *file_name);
+char *file_read_line_alloc(FILE * file);
+int file_link(const char *src, const char *dest);
 int file_copy(const char *src, const char *dest);
 int file_mkdir_hier(const char *path, long mode);
 char *file_md5sum_alloc(const char *file_name);
 char *file_sha256sum_alloc(const char *file_name);
 int rm_r(const char *path);
+int file_decompress(const char *in, const char *out);
+
+/* Buffer size used for extracting files from archives. */
+#define EXTRACT_BUFFER_LEN 0x8000
 
 #ifdef __cplusplus
 }
 #endif
-
-#endif /* FILE_UTIL_H */
+#endif                          /* FILE_UTIL_H */
