@@ -45,11 +45,8 @@ nv_pair_list_elt_t *nv_pair_list_pop(nv_pair_list_t * list);
 char *nv_pair_list_find(nv_pair_list_t * list, char *name);
 
 nv_pair_list_elt_t *nv_pair_list_first(nv_pair_list_t * list);
-nv_pair_list_elt_t *nv_pair_list_prev(nv_pair_list_t * list,
-                                      nv_pair_list_elt_t * node);
 nv_pair_list_elt_t *nv_pair_list_next(nv_pair_list_t * list,
                                       nv_pair_list_elt_t * node);
-nv_pair_list_elt_t *nv_pair_list_last(nv_pair_list_t * list);
 
 #ifdef __cplusplus
 }
