@@ -177,12 +177,14 @@ typedef struct opkg_intercept *opkg_intercept_t;
 static opkg_intercept_t opkg_prep_intercepts(void)
 {
     opkg_intercept_t ctx;
+    const char *oldpath;
     char *newpath;
     char *dtemp;
 
     ctx = xcalloc(1, sizeof(*ctx));
     ctx->oldpath = xstrdup(getenv("PATH"));
-    sprintf_alloc(&newpath, "%s:%s", opkg_config->intercepts_dir, ctx->oldpath);
+    oldpath = ctx->oldpath ? ctx->oldpath : "/usr/sbin:/usr/bin:/sbin:/bin";
+    sprintf_alloc(&newpath, "%s:%s", opkg_config->intercepts_dir, oldpath);
     sprintf_alloc(&ctx->statedir, "%s/opkg-intercept-XXXXXX",
                   opkg_config->tmp_dir);
 
@@ -209,7 +211,11 @@ static int opkg_finalize_intercepts(opkg_intercept_t ctx)
     DIR *dir;
     int err = 0;
 
-    setenv("PATH", ctx->oldpath, 1);
+    if (ctx->oldpath)
+        setenv("PATH", ctx->oldpath, 1);
+    else
+        unsetenv("PATH");
+
     opkg_msg(DEBUG, "Removed intercepts dir from PATH; old PATH=%s\n", ctx->oldpath);
     free(ctx->oldpath);
 
@@ -224,7 +230,7 @@ static int opkg_finalize_intercepts(opkg_intercept_t ctx)
 
             sprintf_alloc(&path, "%s/%s", ctx->statedir, de->d_name);
             if (access(path, X_OK) == 0) {
-                const char *argv[] = { "sh", "-c", path, NULL };
+                const char *argv[] = { "/bin/sh", "-c", path, NULL };
                 opkg_msg(DEBUG, "Run intercepted script %s\n", path);
                 xsystem(argv);
             }
@@ -452,6 +458,31 @@ static int opkg_upgrade_cmd(int argc, char **argv)
     return err;
 }
 
+static int opkg_distupgrade_cmd(int argc, char **argv)
+{
+    int err = 0;
+
+#ifdef HAVE_SOLVER_INTERNAL
+    opkg_msg(ERROR, "dist-upgrade command not available with the internal solver enabled\n");
+    err = -1;
+#else
+    int r;
+
+    signal(SIGINT, sigint_handler);
+
+    pkg_info_preinstall_check();
+
+    err = opkg_solver_distupgrade(argc, argv);
+
+    r = opkg_configure_packages(NULL);
+    if (r != 0)
+        err = -1;
+
+    write_status_files_if_changed();
+#endif
+    return err;
+}
+
 static int opkg_download_cmd(int argc, char **argv)
 {
     int i, err = 0;
@@ -534,7 +565,7 @@ static int opkg_list_installed_cmd(int argc, char **argv)
         pkg_name = argv[0];
     }
     available = pkg_vec_alloc();
-    pkg_hash_fetch_all_installed(available, 0);
+    pkg_hash_fetch_all_installed(available, INSTALLED);
     pkg_vec_sort(available, pkg_compare_names);
     for (i = 0; i < available->len; i++) {
         pkg = available->pkgs[i];
@@ -562,7 +593,7 @@ static int opkg_list_changed_conffiles_cmd(int argc, char **argv)
         pkg_name = argv[0];
     }
     available = pkg_vec_alloc();
-    pkg_hash_fetch_all_installed(available, 0);
+    pkg_hash_fetch_all_installed(available, INSTALLED);
     pkg_vec_sort(available, pkg_compare_names);
     for (i = 0; i < available->len; i++) {
         pkg = available->pkgs[i];
@@ -626,7 +657,7 @@ static int opkg_info_status_cmd(int argc, char **argv, int installed_only)
 
     available = pkg_vec_alloc();
     if (installed_only)
-        pkg_hash_fetch_all_installed(available, 1);
+        pkg_hash_fetch_all_installed(available, INSTALLED_HALF_INSTALLED);
     else
         pkg_hash_fetch_available(available);
 
@@ -638,7 +669,7 @@ static int opkg_info_status_cmd(int argc, char **argv, int installed_only)
 
         pkg_formatted_info(stdout, pkg);
 
-        if (opkg_config->verbosity >= NOTICE) {
+        if (opkg_config->verbosity >= INFO) {
             conffile_list_elt_t *iter;
             for (iter = nv_pair_list_first(&pkg->conffiles); iter;
                  iter = nv_pair_list_next(&pkg->conffiles, iter)) {
@@ -792,8 +823,8 @@ static int opkg_flag_cmd(int argc, char **argv)
 static int opkg_files_cmd(int argc, char **argv)
 {
     pkg_t *pkg;
-    str_list_t *files;
-    str_list_elt_t *iter;
+    file_list_t *files;
+    file_list_elt_t *iter;
     char *pkg_version;
 
     if (argc < 1) {
@@ -812,8 +843,10 @@ static int opkg_files_cmd(int argc, char **argv)
     printf("Package %s (%s) is installed on %s and has the following files:\n",
            pkg->name, pkg_version, pkg->dest->name);
 
-    for (iter = str_list_first(files); iter; iter = str_list_next(files, iter))
-        printf("%s\n", (char *)iter->data);
+    for (iter = file_list_first(files); iter; iter = file_list_next(files, iter)) {
+        file_info_t *info = (file_info_t *)iter->data;
+        printf("%s\n", info->path);
+    }
 
     free(pkg_version);
     pkg_free_installed_files(pkg);
@@ -837,7 +870,7 @@ static int opkg_depends_cmd(int argc, char **argv)
     if (opkg_config->query_all)
         pkg_hash_fetch_available(available_pkgs);
     else
-        pkg_hash_fetch_all_installed(available_pkgs, 0);
+        pkg_hash_fetch_all_installed(available_pkgs, INSTALLED);
 
     for (i = 0; i < argc; i++) {
         for (j = 0; j < available_pkgs->len; j++) {
@@ -919,7 +952,7 @@ static int opkg_what_depends_conflicts_replaces_cmd(enum depend_type what_field_
     if (opkg_config->query_all)
         pkg_hash_fetch_available(available_pkgs);
     else
-        pkg_hash_fetch_all_installed(available_pkgs, 0);
+        pkg_hash_fetch_all_installed(available_pkgs, INSTALLED);
 
     /* mark the root set */
     pkg_vec_clear_marks(available_pkgs);
@@ -1030,7 +1063,7 @@ static int opkg_whatprovides_cmd(int argc, char **argv)
         if (opkg_config->query_all)
             pkg_hash_fetch_available(available_pkgs);
         else
-            pkg_hash_fetch_all_installed(available_pkgs, 0);
+            pkg_hash_fetch_all_installed(available_pkgs, INSTALLED);
         for (i = 0; i < argc; i++) {
             const char *target = argv[i];
             unsigned int j;
@@ -1067,16 +1100,16 @@ static int opkg_search_cmd(int argc, char **argv)
 
     pkg_vec_t *installed;
     pkg_t *pkg;
-    str_list_t *installed_files;
-    str_list_elt_t *iter;
-    char *installed_file;
+    file_list_t *installed_files;
+    file_list_elt_t *iter;
+    file_info_t *installed_file;
 
     if (argc < 1) {
         return -1;
     }
 
     installed = pkg_vec_alloc();
-    pkg_hash_fetch_all_installed(installed, 0);
+    pkg_hash_fetch_all_installed(installed, INSTALLED);
     pkg_vec_sort(installed, pkg_compare_names);
 
     for (i = 0; i < installed->len; i++) {
@@ -1084,10 +1117,10 @@ static int opkg_search_cmd(int argc, char **argv)
 
         installed_files = pkg_get_installed_files(pkg);
 
-        for (iter = str_list_first(installed_files); iter;
-                iter = str_list_next(installed_files, iter)) {
-            installed_file = (char *)iter->data;
-            if (fnmatch(argv[0], installed_file, 0) == 0)
+        for (iter = file_list_first(installed_files); iter;
+                iter = file_list_next(installed_files, iter)) {
+            installed_file = (file_info_t *)iter->data;
+            if (fnmatch(argv[0], installed_file->path, 0) == 0)
                 print_pkg(pkg);
         }
 
@@ -1134,6 +1167,8 @@ static opkg_cmd_t cmds[] = {
         PFM_DESCRIPTION | PFM_SOURCE},
     {"upgrade", 0, (opkg_cmd_fun_t) opkg_upgrade_cmd,
         PFM_DESCRIPTION | PFM_SOURCE},
+    {"dist-upgrade", 0, (opkg_cmd_fun_t) opkg_distupgrade_cmd,
+        PFM_DESCRIPTION | PFM_SOURCE},
     {"list", 0, (opkg_cmd_fun_t) opkg_list_cmd, PFM_SOURCE},
     {"list_installed", 0, (opkg_cmd_fun_t) opkg_list_installed_cmd, PFM_SOURCE},
     {"list-installed", 0, (opkg_cmd_fun_t) opkg_list_installed_cmd, PFM_SOURCE},
