@@ -25,6 +25,7 @@
 #include <libgen.h>
 #include <malloc.h>
 #include <stdlib.h>
+#include <sys/stat.h>
 
 #include "pkg.h"
 
@@ -126,6 +127,10 @@ static void pkg_init(pkg_t * pkg)
     pkg->installed_files_ref_cnt = 0;
     pkg->essential = 0;
     pkg->provided_by_hand = 0;
+
+    if (opkg_config->verbose_status_file) {
+        nv_pair_list_init(&pkg->userfields);
+    }
 }
 
 pkg_t *pkg_new(void)
@@ -240,6 +245,9 @@ void pkg_deinit(pkg_t * pkg)
 
     conffile_list_deinit(&pkg->conffiles);
 
+    if (opkg_config->verbose_status_file)
+        nv_pair_list_deinit(&pkg->userfields);
+
     /* XXX: QUESTION: Is forcing this to 1 correct? I suppose so,
      * since if they are calling deinit, they should know. Maybe do an
      * assertion here instead? */
@@ -392,6 +400,13 @@ int pkg_merge(pkg_t * oldpkg, pkg_t * newpkg)
         oldpkg->installed_size = newpkg->installed_size;
     if (!oldpkg->priority)
         oldpkg->priority = xstrdup(newpkg->priority);
+
+    if (opkg_config->verbose_status_file) {
+        if (nv_pair_list_empty(&oldpkg->userfields)) {
+            list_splice_init(&newpkg->userfields.head, &oldpkg->userfields.head);
+        }
+    }
+
     if (!oldpkg->source)
         oldpkg->source = xstrdup(newpkg->source);
 
@@ -429,34 +444,6 @@ abstract_pkg_t *abstract_pkg_new(void)
     return ab_pkg;
 }
 
-void set_flags_from_control(pkg_t * pkg)
-{
-    char *file_name;
-    FILE *fp;
-    int r;
-
-    sprintf_alloc(&file_name, "%s/%s.control", pkg->dest->info_dir, pkg->name);
-
-    fp = fopen(file_name, "r");
-    if (fp == NULL) {
-        opkg_perror(ERROR, "Failed to open %s", file_name);
-        free(file_name);
-        return;
-    }
-
-    free(file_name);
-
-    r = pkg_parse_from_stream(pkg, fp, PFM_ALL ^ PFM_ESSENTIAL);
-    if (r != 0) {
-        opkg_msg(DEBUG, "Unable to read control file for %s. May be empty.\n",
-                 pkg->name);
-    }
-
-    fclose(fp);
-
-    return;
-}
-
 static const char *pkg_state_want_to_str(pkg_state_want_t sw)
 {
     unsigned int i;
@@ -646,24 +633,36 @@ static void pkg_formatted_field(FILE * fp, pkg_t * pkg, const char *field)
         break;
     case 'e':
     case 'E':
-        if (pkg->essential) {
-            fprintf(fp, "Essential: yes\n");
+        if (strcasecmp(field, "Essential") == 0) {
+            if (pkg->essential) {
+                fprintf(fp, "Essential: yes\n");
+            }
+        } else {
+            goto UNKNOWN_FMT_FIELD;
         }
         break;
     case 'f':
     case 'F':
-        if (pkg->filename) {
-            fprintf(fp, "Filename: %s\n", pkg->filename);
+        if (strcasecmp(field, "Filename") == 0) {
+            if (pkg->filename) {
+                fprintf(fp, "Filename: %s\n", pkg->filename);
+            }
+        } else {
+            goto UNKNOWN_FMT_FIELD;
         }
         break;
     case 'i':
     case 'I':
-        if (strcasecmp(field, "Installed-Size") == 0
-		   && pkg->installed_size) {
-            fprintf(fp, "Installed-Size: %ld\n", pkg->installed_size);
-        } else if (strcasecmp(field, "Installed-Time") == 0
-                   && pkg->installed_time) {
-            fprintf(fp, "Installed-Time: %lu\n", pkg->installed_time);
+        if (strcasecmp(field, "Installed-Size") == 0) {
+            if (pkg->installed_size) {
+                fprintf(fp, "Installed-Size: %ld\n", pkg->installed_size);
+            }
+        } else if (strcasecmp(field, "Installed-Time") == 0) {
+            if (pkg->installed_time) {
+                fprintf(fp, "Installed-Time: %lu\n", pkg->installed_time);
+            }
+        } else {
+            goto UNKNOWN_FMT_FIELD;
         }
         break;
     case 'm':
@@ -783,16 +782,20 @@ static void pkg_formatted_field(FILE * fp, pkg_t * pkg, const char *field)
             if (pkg->tags) {
                 fprintf(fp, "Tags: %s\n", pkg->tags);
             }
+        } else {
+            goto UNKNOWN_FMT_FIELD;
         }
         break;
     case 'v':
     case 'V':
-        {
+        if (strcasecmp(field, "Version") == 0) {
             char *version = pkg_version_str_alloc(pkg);
             if (version == NULL)
                 return;
             fprintf(fp, "Version: %s\n", version);
             free(version);
+        } else {
+            goto UNKNOWN_FMT_FIELD;
         }
         break;
     default:
@@ -805,6 +808,24 @@ static void pkg_formatted_field(FILE * fp, pkg_t * pkg, const char *field)
     opkg_msg(ERROR, "Internal error: field=%s\n", field);
 }
 
+static void pkg_formatted_userfields(FILE *fp, pkg_t *pkg)
+{
+    nv_pair_list_elt_t *iter;
+
+    if (nv_pair_list_empty(&pkg->userfields))
+        return;
+
+    for (iter = nv_pair_list_first(&pkg->userfields); iter;
+                iter = nv_pair_list_next(&pkg->userfields, iter)) {
+        nv_pair_t *uf = (nv_pair_t *)iter->data;
+
+        if (uf->name && uf->value) {
+            fprintf(fp, "%s: %s\n", ((nv_pair_t *)iter->data)->name,
+                    ((nv_pair_t *)iter->data)->value);
+        }
+    }
+}
+
 pkg_state_status_t pkg_state_status_from_str(const char *str)
 {
     unsigned int i;
@@ -843,6 +864,9 @@ void pkg_formatted_info(FILE * fp, pkg_t * pkg)
     pkg_formatted_field(fp, pkg, "Installed-Size");
     pkg_formatted_field(fp, pkg, "Installed-Time");
     pkg_formatted_field(fp, pkg, "Tags");
+    if (opkg_config->verbose_status_file) {
+        pkg_formatted_userfields(fp, pkg);
+    }
     fputs("\n", fp);
 }
 
@@ -861,12 +885,28 @@ void pkg_print_status(pkg_t * pkg, FILE * file)
     pkg_formatted_field(file, pkg, "Replaces");
     pkg_formatted_field(file, pkg, "Conflicts");
     pkg_formatted_field(file, pkg, "Status");
+    if (opkg_config->verbose_status_file) {
+        pkg_formatted_field(file, pkg, "Section");
+    }
     pkg_formatted_field(file, pkg, "Essential");
     pkg_formatted_field(file, pkg, "Architecture");
+    if (opkg_config->verbose_status_file) {
+        pkg_formatted_field(file, pkg, "Maintainer");
+        pkg_formatted_field(file, pkg, "MD5sum");
+        pkg_formatted_field(file, pkg, "Size");
+        pkg_formatted_field(file, pkg, "Filename");
+    }
     pkg_formatted_field(file, pkg, "Conffiles");
+    if (opkg_config->verbose_status_file) {
+        pkg_formatted_field(file, pkg, "Source");
+        pkg_formatted_field(file, pkg, "Description");
+    }
     pkg_formatted_field(file, pkg, "Installed-Size");
     pkg_formatted_field(file, pkg, "Installed-Time");
     pkg_formatted_field(file, pkg, "Auto-Installed");
+    if (opkg_config->verbose_status_file) {
+        pkg_formatted_userfields(file, pkg);
+    }
     fputs("\n", file);
 }
 
@@ -1049,7 +1089,7 @@ char *pkg_version_str_alloc(pkg_t * pkg)
 /*
  * XXX: this should be broken into two functions
  */
-str_list_t *pkg_get_installed_files(pkg_t * pkg)
+file_list_t *pkg_get_installed_files(pkg_t * pkg)
 {
     int err, fd;
     char *list_file_name = NULL;
@@ -1064,7 +1104,7 @@ str_list_t *pkg_get_installed_files(pkg_t * pkg)
         return pkg->installed_files;
     }
 
-    pkg->installed_files = str_list_alloc();
+    pkg->installed_files = file_list_alloc();
 
     /*
      * For installed packages, look at the package.list file in the database.
@@ -1107,7 +1147,7 @@ str_list_t *pkg_get_installed_files(pkg_t * pkg)
             fclose(list_file);
             unlink(list_file_name);
             free(list_file_name);
-            str_list_deinit(pkg->installed_files);
+            file_list_deinit(pkg->installed_files);
             pkg->installed_files = NULL;
             return NULL;
         }
@@ -1127,12 +1167,26 @@ str_list_t *pkg_get_installed_files(pkg_t * pkg)
 
     while (1) {
         char *file_name;
+        char *mode_str;
+        mode_t mode = 0;
+        char *link_target = NULL;
+        char *readlink_buf = NULL;
 
         line = file_read_line_alloc(list_file);
         if (line == NULL) {
             break;
         }
+
+        // <filename>\t<mode>\t<link_target> -- all fields except filename are optional
         file_name = line;
+        mode_str = strchr(line, '\t');
+        if (mode_str) {
+            *mode_str++ = 0;
+            link_target = strchr(mode_str, '\t');
+            if (link_target)
+                *link_target++ = 0;
+            mode = (mode_t)strtoul(mode_str, NULL, 0);
+        }
 
         if (list_from_package) {
             if (*file_name == '.') {
@@ -1144,6 +1198,7 @@ str_list_t *pkg_get_installed_files(pkg_t * pkg)
             sprintf_alloc(&installed_file_name, "%s%s", pkg->dest->root_dir,
                           file_name);
         } else {
+            struct stat file_stat;
             int unmatched_offline_root = opkg_config->offline_root
                     && !str_starts_with(file_name, opkg_config->offline_root);
             if (unmatched_offline_root) {
@@ -1153,9 +1208,15 @@ str_list_t *pkg_get_installed_files(pkg_t * pkg)
                 // already contains root_dir as header -> ABSOLUTE
                 sprintf_alloc(&installed_file_name, "%s", file_name);
             }
+            if (xlstat(installed_file_name, &file_stat) == 0) {
+                mode = file_stat.st_mode;
+                if (S_ISLNK(mode))
+                    link_target = readlink_buf = file_readlink_alloc(installed_file_name);
+            }
         }
-        str_list_append(pkg->installed_files, installed_file_name);
+        file_list_append(pkg->installed_files, installed_file_name, mode, link_target);
         free(installed_file_name);
+        free(readlink_buf);
         free(line);
     }
 
@@ -1182,7 +1243,7 @@ void pkg_free_installed_files(pkg_t * pkg)
         return;
 
     if (pkg->installed_files) {
-        str_list_purge(pkg->installed_files);
+        file_list_purge(pkg->installed_files);
     }
 
     pkg->installed_files = NULL;
@@ -1239,7 +1300,8 @@ int pkg_run_script(pkg_t * pkg, const char *script, const char *args)
 
     /* Installed packages have scripts in pkg->dest->info_dir, uninstalled packages
      * have scripts in pkg->tmp_unpack_dir. */
-    if (pkg->state_status == SS_INSTALLED || pkg->state_status == SS_UNPACKED) {
+    if (pkg->state_status == SS_INSTALLED || pkg->state_status == SS_UNPACKED ||
+        pkg->state_status == SS_HALF_INSTALLED) {
         if (pkg->dest == NULL) {
             opkg_msg(ERROR, "Internal error: %s has a NULL dest.\n", pkg->name);
             return -1;
@@ -1269,7 +1331,7 @@ int pkg_run_script(pkg_t * pkg, const char *script, const char *args)
     sprintf_alloc(&cmd, "%s %s", path, args);
     free(path);
     {
-        const char *argv[] = { "sh", "-c", cmd, NULL };
+        const char *argv[] = { "/bin/sh", "-c", cmd, NULL };
         err = xsystem(argv);
     }
     free(cmd);
@@ -1312,22 +1374,22 @@ void pkg_info_preinstall_check(void)
 
     /* update the file owner data structure */
     opkg_msg(INFO, "Updating file owner list.\n");
-    pkg_hash_fetch_all_installed(installed_pkgs, 0);
+    pkg_hash_fetch_all_installed(installed_pkgs, INSTALLED);
     for (i = 0; i < installed_pkgs->len; i++) {
         pkg_t *pkg = installed_pkgs->pkgs[i];
-        str_list_t *installed_files = pkg_get_installed_files(pkg);     /* this causes installed_files to be cached */
-        str_list_elt_t *iter, *niter;
+        file_list_t *installed_files = pkg_get_installed_files(pkg);    /* this causes installed_files to be cached */
+        file_list_elt_t *iter, *niter;
         if (installed_files == NULL) {
             opkg_msg(ERROR,
                      "Failed to determine installed " "files for pkg %s.\n",
                      pkg->name);
             break;
         }
-        for (iter = str_list_first(installed_files), niter = str_list_next(installed_files, iter);
+        for (iter = file_list_first(installed_files), niter = file_list_next(installed_files, iter);
                 iter;
-                iter = niter, niter = str_list_next(installed_files, iter)) {
-            char *installed_file = (char *)iter->data;
-            file_hash_set_file_owner(installed_file, pkg);
+                iter = niter, niter = file_list_next(installed_files, iter)) {
+            file_info_t *installed_file = (file_info_t *)iter->data;
+            file_hash_set_file_owner(installed_file->path, pkg);
         }
         pkg_free_installed_files(pkg);
     }
@@ -1388,7 +1450,7 @@ int pkg_write_changed_filelists(void)
 
     opkg_msg(INFO, "Saving changed filelists.\n");
 
-    pkg_hash_fetch_all_installed(installed_pkgs, 0);
+    pkg_hash_fetch_all_installed(installed_pkgs, INSTALLED);
     for (i = 0; i < installed_pkgs->len; i++) {
         pkg_t *pkg = installed_pkgs->pkgs[i];
         if (pkg->state_flag & SF_FILELIST_CHANGED) {
