@@ -32,11 +32,17 @@
 #include "xfuncs.h"
 
 int pkg_src_init(pkg_src_t * src, const char *name, const char *base_url,
-                 const char *extra_data, int gzip)
+                 pkg_src_options_t *options, const char *extra_data, int gzip)
 {
     src->gzip = gzip;
     src->name = xstrdup(name);
     src->value = xstrdup(base_url);
+    src->options = xmalloc(sizeof(pkg_src_options_t));
+    if (options)
+       src->options->disable_sig_check = options->disable_sig_check;
+    else
+       src->options->disable_sig_check = 0;
+
     if (extra_data)
         src->extra_data = xstrdup(extra_data);
     else
@@ -48,11 +54,11 @@ void pkg_src_deinit(pkg_src_t * src)
 {
     free(src->name);
     free(src->value);
-    if (src->extra_data)
-        free(src->extra_data);
+    free(src->options);
+    free(src->extra_data);
 }
 
-int pkg_src_download(pkg_src_t * src)
+static int pkg_src_download(pkg_src_t * src)
 {
     int err = 0;
     char *url;
@@ -98,7 +104,7 @@ int pkg_src_download(pkg_src_t * src)
     return err;
 }
 
-int pkg_src_download_signature(pkg_src_t * src)
+static int pkg_src_download_signature(pkg_src_t * src)
 {
     int err = 0;
     char *url;
@@ -112,6 +118,7 @@ int pkg_src_download_signature(pkg_src_t * src)
 
     sprintf_alloc(&sigfile, "%s/%s.%s", opkg_config->lists_dir, src->name,
                   sigext);
+    opkg_msg(DEBUG, "sigfile: %s\n", sigfile);
 
     /* get the url for the sig file */
     if (src->extra_data)        /* debian style? */
@@ -119,6 +126,7 @@ int pkg_src_download_signature(pkg_src_t * src)
                       sigext);
     else
         sprintf_alloc(&url, "%s/Packages.%s", src->value, sigext);
+    opkg_msg(DEBUG, "url: %s\n", url);
 
     err = opkg_download(url, sigfile, NULL, NULL);
     if (err) {
@@ -149,6 +157,9 @@ int pkg_src_verify(pkg_src_t * src)
     sprintf_alloc(&feed, "%s/%s", opkg_config->lists_dir, src->name);
     sprintf_alloc(&sigfile, "%s.%s", feed, sigext);
 
+    opkg_msg(DEBUG, "feed: %s\n", feed);
+    opkg_msg(DEBUG, "sigfile: %s\n", sigfile);
+
     if (!file_exists(sigfile)) {
         opkg_msg(ERROR,
                  "Signature file is missing for %s. "
@@ -184,7 +195,7 @@ int pkg_src_update(pkg_src_t * src)
     if (err)
         return err;
 
-    if (opkg_config->check_signature) {
+    if (opkg_config->check_signature && !(src->options->disable_sig_check)) {
         err = pkg_src_download_signature(src);
         if (err)
             return err;
