@@ -25,19 +25,22 @@
 extern "C" {
 #endif
 
+typedef struct {
+    int disable_sig_check;
+} pkg_src_options_t;
+
 typedef struct {
     char *name;
     char *value;
+    pkg_src_options_t *options;
     char *extra_data;
     int gzip;
 } pkg_src_t;
 
 int pkg_src_init(pkg_src_t * src, const char *name, const char *base_url,
-                 const char *extra_data, int gzip);
+                 pkg_src_options_t *options, const char *extra_data, int gzip);
 void pkg_src_deinit(pkg_src_t * src);
 
-int pkg_src_download(pkg_src_t * src);
-int pkg_src_download_signature(pkg_src_t * src);
 int pkg_src_verify(pkg_src_t * src);
 int pkg_src_update(pkg_src_t * src);
 
