@@ -40,8 +40,10 @@ static void pkg_get_provider_replacees(pkg_t *pkg,
             if (old != replacee) {
                 int installed = (replacee->state_status == SS_INSTALLED)
                         || (replacee->state_status == SS_UNPACKED);
-                if (installed)
-                    pkg_vec_insert(replacees, replacee);
+                if (installed) {
+                    if (!pkg_vec_contains(replacees, replacee))
+                        pkg_vec_insert(replacees, replacee);
+                }
             }
         }
     }
@@ -329,7 +331,8 @@ static int pkg_get_installed_replacees(pkg_t *pkg,
                     continue;
                 if (replacee->state_status == SS_INSTALLED &&
                     version_constraints_satisfied(replaces[i].possibilities[0], pkg)) {
-                    pkg_vec_insert(installed_replacees, replacee);
+                    if (!pkg_vec_contains(installed_replacees, replacee))
+                        pkg_vec_insert(installed_replacees, replacee);
                 }
             }
         }
@@ -406,7 +409,7 @@ static int is_provides_installed(pkg_t *pkg)
     int i, ret = 0;
     pkg_vec_t *available_pkgs = pkg_vec_alloc();
 
-    pkg_hash_fetch_all_installed(available_pkgs, 0);
+    pkg_hash_fetch_all_installed(available_pkgs, INSTALLED_TOBE_INSTALLED);
 
     for (i = 0; i < available_pkgs->len; i++) {
         pkg_t *installed_pkg = available_pkgs->pkgs[i];
@@ -435,7 +438,7 @@ static int calculate_dependencies_for(pkg_t *pkg, pkg_vec_t *pkgs_to_install, pk
     ndepends =
         pkg_hash_fetch_unsatisfied_dependencies(pkg, depends, &unresolved);
 
-    if (unresolved) {
+    if (unresolved && !opkg_config->force_depends) {
         opkg_msg(ERROR, "Cannot satisfy the following dependencies for %s:\n",
                  pkg->name);
         tmp = unresolved;
@@ -446,14 +449,12 @@ static int calculate_dependencies_for(pkg_t *pkg, pkg_vec_t *pkgs_to_install, pk
         }
         free(tmp);
         opkg_message(ERROR, "\n");
-        if (!opkg_config->force_depends) {
-            opkg_msg(INFO,
-                "This could mean that your package list is out of date or that the packages\n"
-                "mentioned above do not yet exist (try 'opkg update'). To proceed in spite\n"
-                "of this problem try again with the '-force-depends' option.\n");
-            err = -1;
-            goto cleanup;
-        }
+        opkg_msg(INFO,
+            "This could mean that your package list is out of date or that the packages\n"
+            "mentioned above do not yet exist (try 'opkg update'). To proceed in spite\n"
+            "of this problem try again with the '-force-depends' option.\n");
+        err = -1;
+        goto cleanup;
     }
 
     if (ndepends <= 0) {
@@ -483,8 +484,15 @@ static int calculate_dependencies_for(pkg_t *pkg, pkg_vec_t *pkgs_to_install, pk
                     goto cleanup;
                 }
                 pkg_vec_insert(dep->wanted_by, pkg);
-                calculate_dependencies_for(dep, pkgs_to_install, replacees, orphans);
+                if (calculate_dependencies_for(dep, pkgs_to_install, replacees, orphans)) {
+                    err = -1;
+                    goto cleanup;
+                }
                 pkg_vec_insert(pkgs_to_install, dep);
+
+                if (opkg_config->download_only)
+                    continue;
+
                 old_pkg = pkg_hash_fetch_installed_by_name(dep->name);
 
                 pkg_get_installed_replacees(dep, replacees);
@@ -528,8 +536,15 @@ static int calculate_dependencies_for(pkg_t *pkg, pkg_vec_t *pkgs_to_install, pk
                 err = -1;
                 goto cleanup;
             }
-            calculate_dependencies_for(dep, pkgs_to_install, replacees, orphans);
+            if (calculate_dependencies_for(dep, pkgs_to_install, replacees, orphans)) {
+                err = -1;
+                goto cleanup;
+            }
             pkg_vec_insert(pkgs_to_install, dep);
+
+            if (opkg_config->download_only)
+                continue;
+
             old_pkg = pkg_hash_fetch_installed_by_name(dep->name);
 
             pkg_get_installed_replacees(dep, replacees);
@@ -562,12 +577,6 @@ int internal_solver_solv(typeId  transactionType, pkg_t *pkg, pkg_vec_t *pkgs_to
         pkg->dest = opkg_config->default_dest;
     }
 
-    /* Provides already installed */
-    if (is_provides_installed(pkg)) {
-        opkg_msg(NOTICE, "Provider for %s already installed \n", pkg->name);
-        return 1;
-    }
-
     /* pkg already installed case */
     if (pkg->state_status == SS_INSTALLED && opkg_config->nodeps == 0) {
         err = calculate_dependencies_for(pkg, pkgs_to_install, replacees, orphans);
@@ -598,6 +607,9 @@ int internal_solver_solv(typeId  transactionType, pkg_t *pkg, pkg_vec_t *pkgs_to
             return -1;
     }
 
+    if (opkg_config->download_only)
+        return 0;
+
     /* add orphans for top level pkg */
     if (old_pkg)
         pkg_get_orphan_dependents(pkg, old_pkg, orphans);
