@@ -417,6 +417,15 @@ int is_pkg_a_provides(const pkg_t *pkg_scout, const pkg_t *pkg)
             opkg_msg(DEBUG2, "Seems I've found a provide %s %s\n",
                      pkg_scout->name, pkg->provides[i]->name);
             return 1;
+        } else {
+            /* if a pkg_scout only Provides one virtual package, then also check if
+             * the pkg_scout Provides is provided by pkg */
+            if ((pkg_scout->provides_count == 2)
+                && strcmp(pkg_scout->provides[1]->name, pkg->provides[i]->name) == 0) { /* Found */
+                opkg_msg(DEBUG2, "Seems I've found a provide %s %s\n",
+                         pkg_scout->name, pkg->provides[i]->name);
+                return 1;
+            }
         }
     }
 
@@ -495,7 +504,7 @@ static void __pkg_hash_fetch_conflictees(pkg_t *pkg,
     int i;
 
     pkg_vec_t *available_pkgs = pkg_vec_alloc();
-    pkg_hash_fetch_all_installed(available_pkgs, 0);
+    pkg_hash_fetch_all_installed(available_pkgs, INSTALLED);
 
     for (i = 0; i < available_pkgs->len; i++) {
         pkg_t *cpkg = available_pkgs->pkgs[i];
