@@ -74,7 +74,8 @@ static libsolv_solver_t *libsolv_solver_new(void);
 static void libsolv_solver_free(libsolv_solver_t *libsolv_solver);
 static void libsolv_solver_add_job(libsolv_solver_t *libsolv_solver,
                                    job_action_t action, const char *pkg_name,
-                                   const char *pkg_version);
+                                   const char *pkg_version,
+                                   version_constraint_t constraint);
 static int libsolv_solver_solve(libsolv_solver_t *libsolv_solver);
 static int libsolv_solver_execute_transaction(libsolv_solver_t *libsolv_solver);
 
@@ -82,6 +83,7 @@ int opkg_solver_install(int num_pkgs, char **pkg_names)
 {
     int i, err;
     char *name, *version;
+    version_constraint_t constraint;
     Dataiterator di;
 
     libsolv_solver_t *solver = libsolv_solver_new();
@@ -95,12 +97,12 @@ int opkg_solver_install(int num_pkgs, char **pkg_names)
     }
 
     for (i = 0; i < num_pkgs; i++) {
-        strip_pkg_name_and_version(pkg_names[i], &name, &version);
+        strip_pkg_name_and_version(pkg_names[i], &name, &version, &constraint);
 
         dataiterator_init(&di, solver->pool, solver->repo_available, 0,
                           SOLVABLE_NAME | SOLVABLE_PROVIDES, name, SEARCH_GLOB);
         while (dataiterator_step(&di)) {
-            libsolv_solver_add_job(solver, JOB_INSTALL, di.kv.str, version);
+            libsolv_solver_add_job(solver, JOB_INSTALL, di.kv.str, version, constraint);
             dataiterator_skip_solvable(&di);
         }
 
@@ -138,7 +140,7 @@ int opkg_solver_remove(int num_pkgs, char **pkg_names)
     for (i = 0; i < num_pkgs; i++){
         dataiterator_init(&di, solver->pool, solver->repo_installed, 0, 0, pkg_names[i], SEARCH_GLOB);
         while (dataiterator_step(&di)) {
-            libsolv_solver_add_job(solver, JOB_REMOVE, di.kv.str, NULL);
+            libsolv_solver_add_job(solver, JOB_REMOVE, di.kv.str, NULL, NONE);
             dataiterator_skip_solvable(&di);
         }
         dataiterator_free(&di);
@@ -165,12 +167,12 @@ int opkg_solver_upgrade(int num_pkgs, char **pkg_names)
         return -1;
 
     if (num_pkgs == 0) {
-        libsolv_solver_add_job(solver, JOB_UPGRADE, 0, NULL);
+        libsolv_solver_add_job(solver, JOB_UPGRADE, 0, NULL, NONE);
     } else {
         for (i = 0; i < num_pkgs; i++) {
             dataiterator_init(&di, solver->pool, solver->repo_installed, 0, 0, pkg_names[i], SEARCH_GLOB);
             while (dataiterator_step(&di)) {
-                libsolv_solver_add_job(solver, JOB_UPGRADE, di.kv.str, NULL);
+                libsolv_solver_add_job(solver, JOB_UPGRADE, di.kv.str, NULL, NONE);
                 dataiterator_skip_solvable(&di);
             }
             dataiterator_free(&di);
@@ -198,10 +200,10 @@ int opkg_solver_distupgrade(int num_pkgs, char **pkg_names)
         return -1;
 
     if (num_pkgs == 0) {
-        libsolv_solver_add_job(solver, JOB_DISTUPGRADE, 0, NULL);
+        libsolv_solver_add_job(solver, JOB_DISTUPGRADE, 0, NULL, NONE);
     } else {
         for (i = 0; i < num_pkgs; i++)
-            libsolv_solver_add_job(solver, JOB_DISTUPGRADE, pkg_names[i], NULL);
+            libsolv_solver_add_job(solver, JOB_DISTUPGRADE, pkg_names[i], NULL, NONE);
     }
 
     err = libsolv_solver_solve(solver);
@@ -291,6 +293,34 @@ static void libsolv_solver_set_arch_policy(libsolv_solver_t *libsolv_solver)
     pool_setarchpolicy(libsolv_solver->pool, arch_policy);
 }
 
+/* Convert an opkg constraint to libsolv flags */
+static int constraint_to_solv_flags(version_constraint_t constraint)
+{
+    int flags = 0;
+
+    switch (constraint) {
+    case EARLIER:
+        flags = REL_LT;
+        break;
+    case EARLIER_EQUAL:
+        flags = REL_LT | REL_EQ;
+        break;
+    case EQUAL:
+        flags = REL_EQ;
+        break;
+    case LATER:
+        flags = REL_GT;
+        break;
+    case LATER_EQUAL:
+        flags = REL_GT | REL_EQ;
+        break;
+    default:
+        break;
+    }
+
+    return flags;
+}
+
 /* This transforms a compound depend [e.g. A (=3.0)|C (>=2.0) ]
    into an id usable by libsolv. */
 static Id dep2id(Pool *pool, compound_depend_t *dep)
@@ -309,26 +339,7 @@ static Id dep2id(Pool *pool, compound_depend_t *dep)
         nameId = pool_str2id(pool, depend->pkg->name, 1);
         if (depend->version) {
             versionId = pool_str2id(pool, depend->version, 1);
-
-            switch (depend->constraint) {
-            case EARLIER:
-                flagId = REL_LT;
-                break;
-            case EARLIER_EQUAL:
-                flagId = REL_LT | REL_EQ;
-                break;
-            case EQUAL:
-                flagId = REL_EQ;
-                break;
-            case LATER:
-                flagId = REL_GT;
-                break;
-            case LATER_EQUAL:
-                flagId = REL_GT | REL_EQ;
-                break;
-            default:
-                break;
-            }
+            flagId = constraint_to_solv_flags(depend->constraint);
             dependId = pool_rel2id(pool, nameId, versionId, flagId, 1);
         } else {
             dependId = nameId;
@@ -464,7 +475,7 @@ static void populate_installed_repo(libsolv_solver_t *libsolv_solver)
 
     pkg_vec_t *installed_pkgs = pkg_vec_alloc();
 
-    pkg_hash_fetch_all_installed(installed_pkgs, 1);
+    pkg_hash_fetch_all_installed(installed_pkgs, INSTALLED_HALF_INSTALLED);
 
     for (i = 0; i < installed_pkgs->len; i++) {
         pkg_t *pkg = installed_pkgs->pkgs[i];
@@ -488,6 +499,9 @@ static void populate_installed_repo(libsolv_solver_t *libsolv_solver)
         if (!pkg->auto_installed)
             queue_push2(&libsolv_solver->solver_jobs, SOLVER_SOLVABLE
                         | SOLVER_USERINSTALLED, solvable_id);
+        else
+            queue_push2(&libsolv_solver->solver_jobs, SOLVER_SOLVABLE
+                        | SOLVER_ALLOWUNINSTALL, solvable_id);
 
         /* if the package is held, mark it as locked */
         if (pkg->state_flag & SF_HOLD)
@@ -569,6 +583,11 @@ static void populate_available_repos(libsolv_solver_t *libsolv_solver)
     pkg_vec_free(available_pkgs);
 }
 
+static void printsolution_callback(struct _Pool *pool, void *data, int type, const char *str)
+{
+    opkg_message(ERROR, "%s\n", str);
+}
+
 static int libsolv_solver_init(libsolv_solver_t *libsolv_solver)
 {
     /* initialize the solver job queue */
@@ -626,9 +645,15 @@ static int libsolv_solver_init(libsolv_solver_t *libsolv_solver)
     if (opkg_config->force_downgrade)
         solver_set_flag(libsolv_solver->solver,
                         SOLVER_FLAG_ALLOW_DOWNGRADE, 1);
-    if (opkg_config->no_install_recommends)
+    if (opkg_config->no_install_recommends) {
         solver_set_flag(libsolv_solver->solver,
                         SOLVER_FLAG_IGNORE_RECOMMENDED, 1);
+    } else {
+         solver_set_flag(libsolv_solver->solver,
+                        SOLVER_FLAG_STRONG_RECOMMENDS, 1);
+         solver_set_flag(libsolv_solver->solver,
+                        SOLVER_FLAG_INSTALL_ALSO_UPDATES, 1);
+    }
     if (!opkg_config->prefer_arch_to_version) {
         solver_set_flag(libsolv_solver->solver,
                         SOLVER_FLAG_ALLOW_ARCHCHANGE, 1);
@@ -636,6 +661,9 @@ static int libsolv_solver_init(libsolv_solver_t *libsolv_solver)
                         SOLVER_FLAG_NO_INFARCHCHECK, 1);
     }
 
+    /* Set callback to log solutions to error queue during solver_printsolution */
+    pool_setdebugcallback(libsolv_solver->pool, printsolution_callback, NULL);
+
     return 0;
 }
 
@@ -657,7 +685,8 @@ static libsolv_solver_t *libsolv_solver_new(void)
 
 static void libsolv_solver_add_job(libsolv_solver_t *libsolv_solver,
                                    job_action_t action, const char *pkg_name,
-                                   const char *pkg_version)
+                                   const char *pkg_version,
+                                   version_constraint_t constraint)
 {
     Id what = 0;
     Id how = 0;
@@ -666,7 +695,7 @@ static void libsolv_solver_add_job(libsolv_solver_t *libsolv_solver,
         what = pool_rel2id(libsolv_solver->pool,
                            pool_str2id(libsolv_solver->pool, pkg_name, 1),
                            pool_str2id(libsolv_solver->pool, pkg_version, 1),
-                           REL_EQ, 1);
+                           constraint_to_solv_flags(constraint), 1);
     } else {
         what = pool_str2id(libsolv_solver->pool, pkg_name, 1);
     }
@@ -689,7 +718,7 @@ static void libsolv_solver_add_job(libsolv_solver_t *libsolv_solver,
         break;
     case JOB_DISTUPGRADE:
         if (pkg_name && strcmp(pkg_name, "") != 0) {
-            how = SOLVER_DISTUPGRADE | SOLVER_SOLVABLE_PROVIDES;
+            how = SOLVER_DISTUPGRADE | SOLVER_SOLVABLE_PROVIDES | SOLVER_TARGETED;
         } else {
             how = SOLVER_DISTUPGRADE | SOLVER_SOLVABLE_ALL;
             queue_push2(&libsolv_solver->solver_jobs, SOLVER_DROP_ORPHANED | SOLVER_SOLVABLE_ALL, 0);
@@ -720,15 +749,15 @@ static int libsolv_solver_solve(libsolv_solver_t *libsolv_solver)
 
     /* print out all problems and recommended solutions */
     if (problem_count) {
-        opkg_message(NOTICE, "Solver encountered %d problem(s):\n", problem_count);
+        opkg_message(ERROR, "Solver encountered %d problem(s):\n", problem_count);
 
         int problem;
         /* problems start at 1, not 0 */
         for (problem = 1; problem <= problem_count; problem++) {
-            opkg_message(NOTICE, "Problem %d/%d:\n", problem, problem_count);
-            opkg_message(NOTICE, "  - %s\n",
+            opkg_message(ERROR, "Problem %d/%d:\n", problem, problem_count);
+            opkg_message(ERROR, "  - %s\n",
                          solver_problem2str(libsolv_solver->solver, problem));
-            opkg_message(NOTICE, "\n");
+            opkg_message(ERROR, "\n");
 
             int solution_count = solver_solution_count(libsolv_solver->solver,
                                                        problem);
@@ -736,7 +765,7 @@ static int libsolv_solver_solve(libsolv_solver_t *libsolv_solver)
 
             /* solutions also start from 1 */
             for (solution = 1; solution <= solution_count; solution++) {
-                opkg_message(NOTICE, "Solution %d:\n", solution);
+                opkg_message(ERROR, "Solution %d:\n", solution);
                 solver_printsolution(libsolv_solver->solver, problem, solution);
                 opkg_message(NOTICE, "\n");
             }
