@@ -46,9 +46,11 @@ enum {
     ARGS_OPT_FORCE_SPACE,
     ARGS_OPT_FORCE_POSTINSTALL,
     ARGS_OPT_FORCE_REMOVE,
+    ARGS_OPT_FORCE_CHECKSUM,
     ARGS_OPT_PREFER_ARCH_TO_VERSION,
     ARGS_OPT_ADD_ARCH,
     ARGS_OPT_ADD_DEST,
+    ARGS_OPT_SIZE,
     ARGS_OPT_ADD_EXCLUDE,
     ARGS_OPT_NOACTION,
     ARGS_OPT_DOWNLOAD_ONLY,
@@ -95,8 +97,10 @@ static struct option long_options[] = {
     {"force_postinstall", 0, 0, ARGS_OPT_FORCE_POSTINSTALL},
     {"force-remove", 0, 0, ARGS_OPT_FORCE_REMOVE},
     {"force_remove", 0, 0, ARGS_OPT_FORCE_REMOVE},
+    {"force-checksum", 0, 0, ARGS_OPT_FORCE_CHECKSUM},
+    {"force_checksum", 0, 0, ARGS_OPT_FORCE_CHECKSUM},
     {"prefer-arch-to-version", 0, 0, ARGS_OPT_PREFER_ARCH_TO_VERSION},
-    {"prefer-arch-to-version", 0, 0, ARGS_OPT_PREFER_ARCH_TO_VERSION},
+    {"prefer_arch_to_version", 0, 0, ARGS_OPT_PREFER_ARCH_TO_VERSION},
     {"noaction", 0, 0, ARGS_OPT_NOACTION},
     {"download-only", 0, 0, ARGS_OPT_DOWNLOAD_ONLY},
     {"nodeps", 0, 0, ARGS_OPT_NODEPS},
@@ -105,10 +109,13 @@ static struct option long_options[] = {
     {"offline-root", 1, 0, 'o'},
     {"add-arch", 1, 0, ARGS_OPT_ADD_ARCH},
     {"add-dest", 1, 0, ARGS_OPT_ADD_DEST},
+    {"size", 0, 0, ARGS_OPT_SIZE},
     {"add-exclude", 1, 0, ARGS_OPT_ADD_EXCLUDE},
     {"test", 0, 0, ARGS_OPT_NOACTION},
     {"tmp-dir", 1, 0, 't'},
     {"tmp_dir", 1, 0, 't'},
+    {"lists-dir", 1, 0, 'l'},
+    {"lists_dir", 1, 0, 'l'},
     {"cache-dir", 1, 0, ARGS_OPT_CACHE_DIR},
     {"host-cache-dir", 0, 0, ARGS_OPT_HOST_CACHE_DIR},
     {"volatile-cache", 0, 0, ARGS_OPT_VOLATILE_CACHE},
@@ -123,9 +130,10 @@ static int args_parse(int argc, char *argv[])
     int option_index = 0;
     int parse_err = 0;
     char *tuple, *targ;
+    char *solver_version = NULL;
 
     while (1) {
-        c = getopt_long_only(argc, argv, "Ad:f:no:p:t:vV::", long_options,
+        c = getopt_long_only(argc, argv, "Ad:f:no:p:l:t:vV::", long_options,
                              &option_index);
         if (c == -1)
             break;
@@ -146,8 +154,17 @@ static int args_parse(int argc, char *argv[])
         case 't':
             opkg_config->tmp_dir = xstrdup(optarg);
             break;
+        case 'l':
+            opkg_config->lists_dir = xstrdup(optarg);
+            break;
         case 'v':
-            printf("opkg version %s\n", VERSION);
+            solver_version = opkg_solver_version_alloc();
+            if (solver_version) {
+                printf("opkg version " VERSION " (%s)\n", solver_version);
+                free(solver_version);
+            } else {
+                printf("opkg version " VERSION "\n");
+            }
             exit(0);
         case 'V':
             opkg_config->verbosity = INFO;
@@ -190,6 +207,9 @@ static int args_parse(int argc, char *argv[])
         case ARGS_OPT_FORCE_REMOVE:
             opkg_config->force_remove = 1;
             break;
+        case ARGS_OPT_FORCE_CHECKSUM:
+            opkg_config->force_checksum = 1;
+            break;
         case ARGS_OPT_PREFER_ARCH_TO_VERSION:
             opkg_config->prefer_arch_to_version = 1;
             break;
@@ -211,6 +231,9 @@ static int args_parse(int argc, char *argv[])
         case ARGS_OPT_ADD_EXCLUDE:
             str_list_append(&opkg_config->exclude_list, optarg);
             break;
+        case ARGS_OPT_SIZE:
+            opkg_config->size = 1;
+            break;
         case ARGS_OPT_NOACTION:
             opkg_config->noaction = 1;
             break;
@@ -271,6 +294,7 @@ static void usage()
     printf("\tlist-changed-conffiles          List user modified configuration files\n");
     printf("\tfiles <pkg>                     List files belonging to <pkg>\n");
     printf("\tsearch <file|glob>              List package providing <file>\n");
+    printf("\tfind <regexp>                   List packages with names or description matching <regexp>\n");
     printf("\tinfo [pkg|glob]                 Display all info for <pkg>\n");
     printf("\tstatus [pkg|glob]               Display all status for <pkg>\n");
     printf("\tdownload <pkg>                  Download <pkg> to current directory\n");
@@ -312,6 +336,7 @@ static void usage()
     printf("\t                                than one candidate is found.\n");
     printf("\t--combine                       Combine upgrade and install operations, this\n");
     printf("\t                                may be needed to resolve dependency issues.\n");
+    printf("\t                                Only available for the internal solver backend.\n");
 
     printf("\nForce Options:\n");
     printf("\t--force-depends                 Install/remove despite failed dependencies\n");
@@ -322,7 +347,9 @@ static void usage()
     printf("\t--force-space                   Disable free space checks\n");
     printf("\t--force-postinstall             Run postinstall scripts even in offline mode\n");
     printf("\t--force-remove                  Remove package even if prerm script fails\n");
+    printf("\t--force-checksum                Don't fail on checksum mismatches\n");
     printf("\t--noaction                      No action -- test only\n");
+    printf("\t--size                          Print package size when listing available packages\n");
     printf("\t--download-only                 No action -- download only\n");
     printf("\t--nodeps                        Do not follow dependencies\n");
     printf("\t--no-install-recommends         Do not install any recommended packages\n");
@@ -330,8 +357,8 @@ static void usage()
     printf("\t                                Remove package and all dependencies\n");
     printf("\t--autoremove                    Remove packages that were installed\n");
     printf("\t                                automatically to satisfy dependencies\n");
-    printf("\t-t                              Specify tmp-dir.\n");
-    printf("\t--tmp-dir                       Specify tmp-dir.\n");
+    printf("\t-t, --tmp-dir <directory>       Specify tmp-dir.\n");
+    printf("\t-l, --lists-dir <directory>     Specify lists-dir.\n");
     printf("\t--cache-dir <path>              Specify cache directory.\n");
     printf("\t--host-cache-dir                Don't place cache in offline root dir.\n");
     printf("\t--volatile-cache                Use volatile cache.\n");
@@ -356,6 +383,7 @@ int main(int argc, char *argv[])
     opkg_cmd_t *cmd;
     int nocheckfordirorfile;
     int noreadfeedsfile;
+    int noloadconf;
 
     if (opkg_conf_init())
         goto err0;
@@ -373,21 +401,24 @@ int main(int argc, char *argv[])
     nocheckfordirorfile = !strcmp(cmd_name, "print-architecture")
         || !strcmp(cmd_name, "print_architecture")
         || !strcmp(cmd_name, "print-installation-architecture")
-        || !strcmp(cmd_name, "print_installation_architecture");
+        || !strcmp(cmd_name, "print_installation_architecture")
+        || !strcmp(cmd_name, "compare_versions")
+        || !strcmp(cmd_name, "compare-versions");
 
     noreadfeedsfile = !strcmp(cmd_name, "flag")
         || !strcmp(cmd_name, "configure")
         || !strcmp(cmd_name, "remove")
         || !strcmp(cmd_name, "files")
         || !strcmp(cmd_name, "search")
-        || !strcmp(cmd_name, "compare_versions")
-        || !strcmp(cmd_name, "compare-versions")
         || !strcmp(cmd_name, "list_installed")
         || !strcmp(cmd_name, "list-installed")
         || !strcmp(cmd_name, "list_changed_conffiles")
         || !strcmp(cmd_name, "list-changed-conffiles")
         || !strcmp(cmd_name, "status");
 
+    noloadconf = !strcmp(cmd_name, "compare_versions")
+        || !strcmp(cmd_name, "compare-versions");
+
     cmd = opkg_cmd_find(cmd_name);
     if (cmd == NULL) {
         fprintf(stderr, "%s: unknown sub-command %s\n", argv[0], cmd_name);
@@ -396,8 +427,10 @@ int main(int argc, char *argv[])
 
     opkg_config->pfm = cmd->pfm;
 
-    if (opkg_conf_load())
-        goto err0;
+    if (!noloadconf) {
+        if (opkg_conf_load())
+            goto err0;
+    }
 
     if (!nocheckfordirorfile) {
         if (!noreadfeedsfile) {
